'use strict';

/**
 * lasso-postcss
 * Process your CSS with PostCSS in lasso+marko
 *
 * @author: Max Milton <max@wearegenki.com>
 *
 * Copyright 2017 We Are Genki
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// -------------------------------------------------------------------------------

// const fs = require('fs');
// const postcss = require('postcss');

// fs.readFile('src/app.css', (err, css) => {
//   postcss()
//     .process(css, { from: 'src/app.css', to: 'dest/app.css' })
//     .then(result => {
//       fs.writeFile('dest/app.css', result.css);
//       if (result.map) fs.writeFile('dest/app.css.map', result.map);
//     });
// });

// -------------------------------------------------------------------------------

// const autoprefixer = require('autoprefixer');
// const postcss = require('postcss');
// const DEFAULT_OPTS = {};

// module.exports = function (lasso, sourceMap, options) {
//   // use DEFAULT_OPTS if options not present
//   options = options || DEFAULT_OPTS;

//   lasso.addTransform({
//     contentType: 'css',
//     name: module.id,
//     stream: false,
//     transform: function (code) {
//       const processed = postcss([autoprefixer(options)]).process(code);

//       processed.warnings().forEach(function (warn) {
//         process.stderr.write(warn.toString());
//       });
//       return processed.css;
//     }
//   });
// };

// -------------------------------------------------------------------------------

const postcss = require('postcss');
const loadConfig = require('postcss-load-config');
const path = require('path');

function withConfigLoader(cb) {
  return (plugins, options) => {
    if (Array.isArray(plugins)) {
      return cb(() => Promise.resolve({
        plugins,
        options,
      }));
    } else if (typeof plugins === 'function') {
      return cb(file => Promise.resolve(plugins(file)));
    }

    const contextOptions = plugins || {};

    return cb((file) => {
      let configPath;

      if (contextOptions.config) {
        if (path.isAbsolute(contextOptions.config)) {
          configPath = contextOptions.config;
        } else {
          configPath = path.join(file.base, contextOptions.config);
        }
      } else {
        configPath = file.dirname;
      }

      return loadConfig({ file, options: contextOptions }, configPath);
    });
  };
}

// module.exports = withConfigLoader((config) => {
//   if (file.isNull()) {
//     return cb(null, file)
//   }

//   var options = {
//     from: file.path,
//     to: file.path,
//     // Generate a separate source map for gulp-sourcemaps
//     map: file.sourceMap ? { annotation: false } : false
//   }

//   loadConfig(file)
//     .then(function (config) {
//       var configOpts = config.options || {}
//       // Extend the default options if not protected
//       for (var opt in configOpts) {
//         if (configOpts.hasOwnProperty(opt) && !isProtected[opt]) {
//           options[opt] = configOpts[opt]
//         } else {
//           gutil.log(
//             'gulp-postcss:',
//             file.relative + '\nCannot override ' + opt +
//             ' option, because it is required by gulp-sourcemaps'
//           )
//         }
//       }
//       return postcss(config.plugins || [])
//         .process(file.contents, options)
//     })
//     .then(handleResult, handleError)

//   function handleResult(result) {
//     var map
//     var warnings = result.warnings().join('\n')

//     file.contents = new Buffer(result.css)

//     // Apply source map to the chain
//     if (file.sourceMap) {
//       map = result.map.toJSON()
//       map.file = file.relative
//       map.sources = [].map.call(map.sources, function (source) {
//         return path.join(path.dirname(file.relative), source)
//       })
//       applySourceMap(file, map)
//     }

//     if (warnings) {
//       gutil.log('gulp-postcss:', file.relative + '\n' + warnings)
//     }

//     setImmediate(function () {
//       cb(null, file)
//     })
//   }

//   function handleError(error) {
//     var errorOptions = { fileName: file.path, showStack: true }
//     if (error.name === 'CssSyntaxError') {
//       errorOptions.error = error
//       errorOptions.fileName = error.file || file.path
//       errorOptions.lineNumber = error.line
//       errorOptions.showProperties = false
//       errorOptions.showStack = false
//       error = error.message + '\n\n' + error.showSourceCode() + '\n'
//     }
//     // Prevent stream’s unhandled exception from
//     // being suppressed by Promise
//     setImmediate(function () {
//       cb(new gutil.PluginError('gulp-postcss', error, errorOptions))
//     })
// });

module.exports = (lasso, config) => {
  // const options = config.options || {};

  // lasso.addTransform({
  //   contentType: 'css',
  //   name: module.id,
  //   stream: false,
  //   transform: (code) => {
  //     const processed = postcss([autoprefixer(options)]).process(code);

  //     processed.warnings().forEach((warn) => {
  //       process.stderr.write(warn.toString());
  //     });
  //     return processed.css;
  //   },
  // });

  config.less = config.less || less;

  const extensions = config.extensions || ['css', 'pcss', 'postcss'];
  extensions.forEach((ext) => {
    const type = ext.charAt(0) === '.' ? ext.substring(1) : ext;
    lasso.dependencies.registerStyleSheetType(type, require('./dependency-less')(config, lasso));
  });

  lasso.dependencies.registerPackageType('less-import', require('./dependency-less-import')(config, lasso));
};
